// LinearSet.java, created Wed Aug  4 12:03:54 1999 by pnkfelix
// Copyright (C) 1999 Felix S. Klock II <pnkfelix@mit.edu>
// Licensed under the terms of the GNU GPL; see COPYING for details.
package net.cscott.jutil;

import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * {@link LinearSet} is a simplistic light-weight
 * {@link Set} designed for use when the number of entries is
 * small.  It is backed by a {@link List}.
 * 
 * @author  Felix S. Klock II <pnkfelix@mit.edu>
 * @version $Id: LinearSet.java,v 1.6 2006-10-30 19:58:06 cananian Exp $
 */
public class LinearSet<E> extends AbstractSet<E> implements Cloneable,
						      java.io.Serializable {
    private List<E> list;
    private ListFactory<E> lf;

    /** Creates a {@link LinearSet}. Uses an
	{@link ArrayList} as the backing store.*/
    public LinearSet() {
        this(Factories.<E>arrayListFactory());
    }

    /** Creates a {@link LinearSet} with given capacity. 
	Uses an {@link ArrayList} as the backing store.
     */
    public LinearSet(final int capacity) {
	this(Factories.<E>arrayListFactory(), capacity);
    }

    /** Creates a {@link LinearSet}, filling it with the elements
	of <code>set</code>.  Uses an {@link ArrayList} as the
	backing store.
    */
    public LinearSet(final Set<? extends E> set) {
	this(Factories.<E>arrayListFactory(), set);
    }

    /** Creates a {@link LinearSet}, filling it with the elements
	of the given collection.  Uses an {@link ArrayList} as the
	backing store.  Uses time quadratic in the size of the given
	collection.
    */
    public LinearSet(final Collection<? extends E> c) {
	this(Factories.<E>arrayListFactory());
	addAll(c);
    }

    /** Creates an empty {@link LinearSet}, using a
	{@link List} generated by <code>lf</code> as the backing
	store.  
    */
    public LinearSet(final ListFactory<E> lf) {
	list = lf.makeList();
	this.lf = lf;
    }

    /** Creates an empty {@link LinearSet} with a given capacity,
	using a {@link List} generated by <code>lf</code> as the
	backing store.  
    */
    public LinearSet(final ListFactory<E> lf, int capacity) {
	list = lf.makeList(capacity);
	this.lf = lf;
    }

    /** Creates an empty {@link LinearSet}, using a
	{@link List} generated by <code>lf</code> as the backing
	store, and fills it with the elements of <code>set</code>.
    */
    public LinearSet(final ListFactory<E> lf, final Set<? extends E> set) {
	list = lf.makeList(set);
	this.lf = lf;
    }

    public Iterator<E> iterator() {
	return list.iterator();
    }

    public int size() {
	return list.size();
    }

    public boolean add(E o) {
	if (list.contains(o)) {
	    return false;
	} else {
	    list.add(o);
	    return true;
	}
    }

    public boolean addAll(Collection<? extends E> c) {
	HashSet<E> s = new HashSet<E>(this.size() + c.size());
	s.addAll(this);
	boolean r = s.addAll(c);
	this.list = lf.makeList(s);
	return r;
    }

    public boolean remove(Object o) {
	int index = list.indexOf(o);
	if (index == -1) {
	    return false;
	} else {
	    list.remove(index);
	    return true;
	}
    }

    public LinearSet<E> clone() {
	return new LinearSet<E>(this);
    }
}
